/* Copyright (c) 2001, Stanford University
	All rights reserved.

	See the file LICENSE.txt for information on redistributing this software. */
	

#include "cr_spu.h"
#include "chromium.h"
#include "cr_error.h"
#include "cr_mem.h"
#include "cr_net.h"
#include "server_dispatch.h"
#include "server.h"

static unsigned int lookupComponents( GLenum pname )
{
	switch( pname )
	{

			case GL_AMBIENT: return 4;
			case GL_COLOR_INDEXES: return 3;
			case GL_CONSTANT_ATTENUATION: return 1;
			case GL_DIFFUSE: return 4;
			case GL_EMISSION: return 4;
			case GL_EYE_PLANE: return 4;
			case GL_LINEAR_ATTENUATION: return 1;
			case GL_OBJECT_PLANE: return 4;
			case GL_POSITION: return 4;
			case GL_QUADRATIC_ATTENUATION: return 1;
			case GL_SHININESS: return 1;
			case GL_SPECULAR: return 4;
			case GL_SPOT_CUTOFF: return 1;
			case GL_SPOT_DIRECTION: return 3;
			case GL_SPOT_EXPONENT: return 1;
			case GL_TEXTURE_ALPHA_SIZE: return 1;
			case GL_TEXTURE_BLUE_SIZE: return 1;
			case GL_TEXTURE_BORDER: return 1;
			case GL_TEXTURE_BORDER_COLOR: return 4;
			case GL_TEXTURE_COMPONENTS: return 1;
			case GL_TEXTURE_DEPTH: return 1;
			case GL_TEXTURE_ENV_COLOR: return 4;
			case GL_TEXTURE_ENV_MODE: return 1;
			case GL_TEXTURE_GEN_MODE: return 1;
			case GL_TEXTURE_GREEN_SIZE: return 1;
			case GL_TEXTURE_HEIGHT: return 1;
			case GL_TEXTURE_INTENSITY_SIZE: return 1;
			case GL_TEXTURE_LUMINANCE_SIZE: return 1;
			case GL_TEXTURE_MAG_FILTER: return 1;
			case GL_TEXTURE_MIN_FILTER: return 1;
			case GL_TEXTURE_RED_SIZE: return 1;
			case GL_TEXTURE_RESIDENT: return 1;
			case GL_TEXTURE_WIDTH: return 1;
			case GL_TEXTURE_WRAP_S: return 1;
			case GL_TEXTURE_WRAP_T: return 1;
#ifdef CR_ARB_texture_env_combine
			case GL_ALPHA_SCALE: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_ARB_vertex_buffer_object
			case GL_BUFFER_ACCESS_ARB: return 1;
#endif /* CR_ARB_vertex_buffer_object */
#ifdef CR_ARB_vertex_buffer_object
			case GL_BUFFER_MAPPED_ARB: return 1;
#endif /* CR_ARB_vertex_buffer_object */
#ifdef CR_ARB_vertex_buffer_object
			case GL_BUFFER_SIZE_ARB: return 1;
#endif /* CR_ARB_vertex_buffer_object */
#ifdef CR_ARB_vertex_buffer_object
			case GL_BUFFER_USAGE_ARB: return 1;
#endif /* CR_ARB_vertex_buffer_object */
#ifdef CR_NV_register_combiners
			case GL_COMBINER_AB_DOT_PRODUCT_NV: return 1;
#endif /* CR_NV_register_combiners */
#ifdef CR_NV_register_combiners
			case GL_COMBINER_AB_OUTPUT_NV: return 1;
#endif /* CR_NV_register_combiners */
#ifdef CR_NV_register_combiners
			case GL_COMBINER_BIAS_NV: return 1;
#endif /* CR_NV_register_combiners */
#ifdef CR_NV_register_combiners
			case GL_COMBINER_CD_DOT_PRODUCT_NV: return 1;
#endif /* CR_NV_register_combiners */
#ifdef CR_NV_register_combiners
			case GL_COMBINER_CD_OUTPUT_NV: return 1;
#endif /* CR_NV_register_combiners */
#ifdef CR_NV_register_combiners
			case GL_COMBINER_COMPONENT_USAGE_NV: return 1;
#endif /* CR_NV_register_combiners */
#ifdef CR_NV_register_combiners
			case GL_COMBINER_INPUT_NV: return 1;
#endif /* CR_NV_register_combiners */
#ifdef CR_NV_register_combiners
			case GL_COMBINER_MAPPING_NV: return 1;
#endif /* CR_NV_register_combiners */
#ifdef CR_NV_register_combiners
			case GL_COMBINER_MUX_SUM_NV: return 1;
#endif /* CR_NV_register_combiners */
#ifdef CR_NV_register_combiners
			case GL_COMBINER_SCALE_NV: return 1;
#endif /* CR_NV_register_combiners */
#ifdef CR_NV_register_combiners
			case GL_COMBINER_SUM_OUTPUT_NV: return 1;
#endif /* CR_NV_register_combiners */
#ifdef CR_ARB_texture_env_combine
			case GL_COMBINE_ALPHA_ARB: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_ARB_texture_env_combine
			case GL_COMBINE_RGB_ARB: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_NV_register_combiners
			case GL_CONSTANT_COLOR0_NV: return 4;
#endif /* CR_NV_register_combiners */
#ifdef CR_NV_register_combiners
			case GL_CONSTANT_COLOR1_NV: return 4;
#endif /* CR_NV_register_combiners */
#ifdef CR_ARB_point_sprite
			case GL_COORD_REPLACE_ARB: return 1;
#endif /* CR_ARB_point_sprite */
#ifdef CR_ARB_occlusion_query
			case GL_CURRENT_QUERY_ARB: return 1;
#endif /* CR_ARB_occlusion_query */
#ifdef CR_any_vertex_program
			case GL_CURRENT_VERTEX_ATTRIB_ARB: return 4;
#endif /* CR_any_vertex_program */
#ifdef CR_ARB_depth_texture
			case GL_DEPTH_TEXTURE_MODE_ARB: return 1;
#endif /* CR_ARB_depth_texture */
#ifdef CR_SGIS_generate_mipmap
			case GL_GENERATE_MIPMAP_SGIS: return 1;
#endif /* CR_SGIS_generate_mipmap */
#ifdef CR_ARB_texture_env_combine
			case GL_OPERAND0_ALPHA_ARB: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_ARB_texture_env_combine
			case GL_OPERAND0_RGB_ARB: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_ARB_texture_env_combine
			case GL_OPERAND1_ALPHA_ARB: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_ARB_texture_env_combine
			case GL_OPERAND1_RGB_ARB: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_ARB_texture_env_combine
			case GL_OPERAND2_ALPHA_ARB: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_ARB_texture_env_combine
			case GL_OPERAND2_RGB_ARB: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_ARB_occlusion_query
			case GL_QUERY_COUNTER_BITS_ARB: return 1;
#endif /* CR_ARB_occlusion_query */
#ifdef CR_ARB_occlusion_query
			case GL_QUERY_RESULT_ARB: return 1;
#endif /* CR_ARB_occlusion_query */
#ifdef CR_ARB_occlusion_query
			case GL_QUERY_RESULT_AVAILABLE_ARB: return 1;
#endif /* CR_ARB_occlusion_query */
#ifdef CR_ARB_texture_env_combine
			case GL_RGB_SCALE_ARB: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_ARB_texture_env_combine
			case GL_SOURCE0_ALPHA_ARB: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_ARB_texture_env_combine
			case GL_SOURCE0_RGB_ARB: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_ARB_texture_env_combine
			case GL_SOURCE1_ALPHA_ARB: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_ARB_texture_env_combine
			case GL_SOURCE1_RGB_ARB: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_ARB_texture_env_combine
			case GL_SOURCE2_ALPHA_ARB: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_ARB_texture_env_combine
			case GL_SOURCE2_RGB_ARB: return 1;
#endif /* CR_ARB_texture_env_combine */
#ifdef CR_OPENGL_VERSION_1_2
			case GL_TEXTURE_BASE_LEVEL: return 1;
#endif /* CR_OPENGL_VERSION_1_2 */
#ifdef CR_ARB_shadow_ambient
			case GL_TEXTURE_COMPARE_FAIL_VALUE_ARB: return 1;
#endif /* CR_ARB_shadow_ambient */
#ifdef CR_ARB_shadow
			case GL_TEXTURE_COMPARE_FUNC_ARB: return 1;
#endif /* CR_ARB_shadow */
#ifdef CR_ARB_shadow
			case GL_TEXTURE_COMPARE_MODE_ARB: return 1;
#endif /* CR_ARB_shadow */
#ifdef CR_ARB_texture_compression
			case GL_TEXTURE_COMPRESSED: return 1;
#endif /* CR_ARB_texture_compression */
#ifdef CR_ARB_texture_compression
			case GL_TEXTURE_COMPRESSED_IMAGE_SIZE: return 1;
#endif /* CR_ARB_texture_compression */
#ifdef CR_ARB_depth_texture
			case GL_TEXTURE_DEPTH_SIZE_ARB: return 1;
#endif /* CR_ARB_depth_texture */
#ifdef CR_EXT_texture_lod_bias
			case GL_TEXTURE_LOD_BIAS_EXT: return 1;
#endif /* CR_EXT_texture_lod_bias */
#ifdef CR_EXT_texture_filter_anisotropic
			case GL_TEXTURE_MAX_ANISOTROPY_EXT: return 1;
#endif /* CR_EXT_texture_filter_anisotropic */
#ifdef CR_OPENGL_VERSION_1_2
			case GL_TEXTURE_MAX_LEVEL: return 1;
#endif /* CR_OPENGL_VERSION_1_2 */
#ifdef CR_OPENGL_VERSION_1_2
			case GL_TEXTURE_MAX_LOD: return 1;
#endif /* CR_OPENGL_VERSION_1_2 */
#ifdef CR_OPENGL_VERSION_1_2
			case GL_TEXTURE_MIN_LOD: return 1;
#endif /* CR_OPENGL_VERSION_1_2 */
#ifdef CR_OPENGL_VERSION_1_2
			case GL_TEXTURE_PRIORITY: return 1;
#endif /* CR_OPENGL_VERSION_1_2 */
#ifdef CR_OPENGL_VERSION_1_2
			case GL_TEXTURE_WRAP_R: return 1;
#endif /* CR_OPENGL_VERSION_1_2 */
#ifdef CR_any_vertex_program
			case GL_TRACK_MATRIX_NV: return 24;
#endif /* CR_any_vertex_program */
#ifdef CR_any_vertex_program
			case GL_TRACK_MATRIX_TRANSFORM_NV: return 24;
#endif /* CR_any_vertex_program */
#ifdef CR_ARB_vertex_buffer_object
			case GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB: return 1;
#endif /* CR_ARB_vertex_buffer_object */
#ifdef CR_any_vertex_program
			case GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB: return 1;
#endif /* CR_any_vertex_program */
#ifdef CR_any_vertex_program
			case GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB: return 1;
#endif /* CR_any_vertex_program */
#ifdef CR_any_vertex_program
			case GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB: return 1;
#endif /* CR_any_vertex_program */
#ifdef CR_any_vertex_program
			case GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB: return 1;
#endif /* CR_any_vertex_program */
#ifdef CR_any_vertex_program
			case GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB: return 1;
#endif /* CR_any_vertex_program */
#ifdef CR_any_vertex_program
			case GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB: return 1;
#endif /* CR_any_vertex_program */

		default:
			crError( "Unknown parameter name in lookupComponents: %d", (int) pname );
			break;
	}
	/* NOTREACHED */
	return 0;
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetBufferParameterivARB( GLenum target, GLenum pname, GLint * params )
{
	GLint local_params[1];
	(void) params;
	cr_server.head_spu->dispatch_table.GetBufferParameterivARB( target, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetClipPlane( GLenum plane, GLdouble * equation )
{
	GLdouble local_equation[4];
	(void) equation;
	cr_server.head_spu->dispatch_table.GetClipPlane( plane, local_equation );
	crServerReturnValue( &(local_equation[0]), 4*sizeof(GLdouble) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetCombinerInputParameterfvNV( GLenum stage, GLenum portion, GLenum variable, GLenum pname, GLfloat * params )
{
	GLfloat local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetCombinerInputParameterfvNV( stage, portion, variable, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetCombinerInputParameterivNV( GLenum stage, GLenum portion, GLenum variable, GLenum pname, GLint * params )
{
	GLint local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetCombinerInputParameterivNV( stage, portion, variable, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetCombinerOutputParameterfvNV( GLenum stage, GLenum portion, GLenum pname, GLfloat * params )
{
	GLfloat local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetCombinerOutputParameterfvNV( stage, portion, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetCombinerOutputParameterivNV( GLenum stage, GLenum portion, GLenum pname, GLint * params )
{
	GLint local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetCombinerOutputParameterivNV( stage, portion, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetCombinerStageParameterfvNV( GLenum stage, GLenum pname, GLfloat * params )
{
	GLfloat local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetCombinerStageParameterfvNV( stage, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetFenceivNV( GLuint fence, GLenum pname, GLint * params )
{
	GLint local_params[1];
	(void) params;
	cr_server.head_spu->dispatch_table.GetFenceivNV( fence, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetFinalCombinerInputParameterfvNV( GLenum variable, GLenum pname, GLfloat * params )
{
	GLfloat local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetFinalCombinerInputParameterfvNV( variable, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetFinalCombinerInputParameterivNV( GLenum variable, GLenum pname, GLint * params )
{
	GLint local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetFinalCombinerInputParameterivNV( variable, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetLightfv( GLenum light, GLenum pname, GLfloat * params )
{
	GLfloat local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetLightfv( light, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetLightiv( GLenum light, GLenum pname, GLint * params )
{
	GLint local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetLightiv( light, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetMaterialfv( GLenum face, GLenum pname, GLfloat * params )
{
	GLfloat local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetMaterialfv( face, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetMaterialiv( GLenum face, GLenum pname, GLint * params )
{
	GLint local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetMaterialiv( face, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetProgramEnvParameterdvARB( GLenum target, GLuint index, GLdouble * params )
{
	GLdouble local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetProgramEnvParameterdvARB( target, index, local_params );
	crServerReturnValue( &(local_params[0]), 4*sizeof(GLdouble) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetProgramEnvParameterfvARB( GLenum target, GLuint index, GLfloat * params )
{
	GLfloat local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetProgramEnvParameterfvARB( target, index, local_params );
	crServerReturnValue( &(local_params[0]), 4*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetProgramLocalParameterdvARB( GLenum target, GLuint index, GLdouble * params )
{
	GLdouble local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetProgramLocalParameterdvARB( target, index, local_params );
	crServerReturnValue( &(local_params[0]), 4*sizeof(GLdouble) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetProgramLocalParameterfvARB( GLenum target, GLuint index, GLfloat * params )
{
	GLfloat local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetProgramLocalParameterfvARB( target, index, local_params );
	crServerReturnValue( &(local_params[0]), 4*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetProgramNamedParameterdvNV( GLuint id, GLsizei len, const GLubyte * name, GLdouble * params )
{
	GLdouble local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetProgramNamedParameterdvNV( id, len, name, local_params );
	crServerReturnValue( &(local_params[0]), 4*sizeof(GLdouble) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetProgramNamedParameterfvNV( GLuint id, GLsizei len, const GLubyte * name, GLfloat * params )
{
	GLfloat local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetProgramNamedParameterfvNV( id, len, name, local_params );
	crServerReturnValue( &(local_params[0]), 4*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetProgramParameterdvNV( GLenum target, GLuint index, GLenum pname, GLdouble * params )
{
	GLdouble local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetProgramParameterdvNV( target, index, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLdouble) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetProgramParameterfvNV( GLenum target, GLuint index, GLenum pname, GLfloat * params )
{
	GLfloat local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetProgramParameterfvNV( target, index, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetProgramiv( GLuint program, GLenum pname, GLint * params )
{
	GLint local_params[1];
	(void) params;
	cr_server.head_spu->dispatch_table.GetProgramiv( program, pname, local_params );
	crServerReturnValue( &(local_params[0]), 1*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetProgramivARB( GLenum target, GLenum pname, GLint * params )
{
	GLint local_params[1];
	(void) params;
	cr_server.head_spu->dispatch_table.GetProgramivARB( target, pname, local_params );
	crServerReturnValue( &(local_params[0]), 1*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetProgramivNV( GLuint id, GLenum pname, GLint * params )
{
	GLint local_params[1];
	(void) params;
	cr_server.head_spu->dispatch_table.GetProgramivNV( id, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetQueryObjectivARB( GLuint id, GLenum pname, GLint * params )
{
	GLint local_params[1];
	(void) params;
	cr_server.head_spu->dispatch_table.GetQueryObjectivARB( id, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetQueryObjectuivARB( GLuint id, GLenum pname, GLuint * params )
{
	GLuint local_params[1];
	(void) params;
	cr_server.head_spu->dispatch_table.GetQueryObjectuivARB( id, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLuint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetQueryivARB( GLenum target, GLenum pname, GLint * params )
{
	GLint local_params[1];
	(void) params;
	cr_server.head_spu->dispatch_table.GetQueryivARB( target, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetRenderbufferParameterivEXT( GLenum target, GLenum pname, GLint * params )
{
	GLint local_params[1];
	(void) params;
	cr_server.head_spu->dispatch_table.GetRenderbufferParameterivEXT( target, pname, local_params );
	crServerReturnValue( &(local_params[0]), 1*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetShaderiv( GLuint shader, GLenum pname, GLint * params )
{
	GLint local_params[1];
	(void) params;
	cr_server.head_spu->dispatch_table.GetShaderiv( shader, pname, local_params );
	crServerReturnValue( &(local_params[0]), 1*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetTexEnvfv( GLenum target, GLenum pname, GLfloat * params )
{
	GLfloat local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetTexEnvfv( target, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetTexEnviv( GLenum target, GLenum pname, GLint * params )
{
	GLint local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetTexEnviv( target, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetTexGendv( GLenum coord, GLenum pname, GLdouble * params )
{
	GLdouble local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetTexGendv( coord, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLdouble) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetTexGenfv( GLenum coord, GLenum pname, GLfloat * params )
{
	GLfloat local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetTexGenfv( coord, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetTexGeniv( GLenum coord, GLenum pname, GLint * params )
{
	GLint local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetTexGeniv( coord, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetTexLevelParameterfv( GLenum target, GLint level, GLenum pname, GLfloat * params )
{
	GLfloat local_params[1];
	(void) params;
	cr_server.head_spu->dispatch_table.GetTexLevelParameterfv( target, level, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetTexLevelParameteriv( GLenum target, GLint level, GLenum pname, GLint * params )
{
	GLint local_params[1];
	(void) params;
	cr_server.head_spu->dispatch_table.GetTexLevelParameteriv( target, level, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetTexParameterfv( GLenum target, GLenum pname, GLfloat * params )
{
	GLfloat local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetTexParameterfv( target, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetTexParameteriv( GLenum target, GLenum pname, GLint * params )
{
	GLint local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetTexParameteriv( target, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetTrackMatrixivNV( GLenum target, GLuint address, GLenum pname, GLint * params )
{
	GLint local_params[1];
	(void) params;
	cr_server.head_spu->dispatch_table.GetTrackMatrixivNV( target, address, pname, local_params );
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetVertexAttribdvARB( GLuint index, GLenum pname, GLdouble * params )
{
	GLdouble local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetVertexAttribdvARB( index, pname, local_params );
	if (pname==GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB){
		local_params[0]=(GLdouble)crStateBufferHWIDtoID((GLint)local_params[0]);
	}
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLdouble) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetVertexAttribdvNV( GLuint index, GLenum pname, GLdouble * params )
{
	GLdouble local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetVertexAttribdvNV( index, pname, local_params );
	if (pname==GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB){
		local_params[0]=(GLdouble)crStateBufferHWIDtoID((GLint)local_params[0]);
	}
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLdouble) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetVertexAttribfvARB( GLuint index, GLenum pname, GLfloat * params )
{
	GLfloat local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetVertexAttribfvARB( index, pname, local_params );
	if (pname==GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB){
		local_params[0]=(GLfloat)crStateBufferHWIDtoID((GLint)local_params[0]);
	}
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetVertexAttribfvNV( GLuint index, GLenum pname, GLfloat * params )
{
	GLfloat local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetVertexAttribfvNV( index, pname, local_params );
	if (pname==GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB){
		local_params[0]=(GLfloat)crStateBufferHWIDtoID((GLint)local_params[0]);
	}
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLfloat) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetVertexAttribivARB( GLuint index, GLenum pname, GLint * params )
{
	GLint local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetVertexAttribivARB( index, pname, local_params );
	if (pname==GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB){
		local_params[0]=(GLint)crStateBufferHWIDtoID((GLint)local_params[0]);
	}
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

void SERVER_DISPATCH_APIENTRY crServerDispatchGetVertexAttribivNV( GLuint index, GLenum pname, GLint * params )
{
	GLint local_params[4];
	(void) params;
	cr_server.head_spu->dispatch_table.GetVertexAttribivNV( index, pname, local_params );
	if (pname==GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB){
		local_params[0]=(GLint)crStateBufferHWIDtoID((GLint)local_params[0]);
	}
	crServerReturnValue( &(local_params[0]), lookupComponents(pname)*sizeof(GLint) );
}

